from pytest_influxdb.test_result_dto import TestResultDTO


class DataManager:
    def get_test_result_dto(self, test_name, session_dict):
        test_result_dto_dict = session_dict.get('test_result_dto_session_dict')
        test_result_dto_dict.update({test_name: TestResultDTO()})
        test_result_dto: TestResultDTO = test_result_dto_dict.get(test_name)
        test_result_dto.set_test(test_name)
        return test_result_dto

    def get_report(self, request):
        try:
            report_outcome = request.node.rep_call
        except AttributeError:
            report_outcome = request.node.rep_setup
        return report_outcome

    def get_test_influxdb_components(self, influxdb, session_dict):
        influxdb_components = session_dict.get('influxdb')
        if not influxdb_components:
            session_dict.update({'influxdb': influxdb})
            influxdb_components = session_dict.get('influxdb')
        return influxdb_components

    def save_db_data_in_properties(self, request, db_host, db_port, db_user, db_password, db_name,
                                   influxdb_values, run, project, version):
        request.node.user_properties = ({"influxdb": dict(influxdb_host=db_host, influxdb_port=db_port,
                                                          influxdb_user=db_user, influxdb_password=db_password,
                                                          influxdb_name=db_name, influxdb_values=influxdb_values,
                                                          run=run, project=project, version=version)},)

    def get_test_influxdb_credentials(self, terminalreporter):
        influxdb_creds_dict = dict()
        for test_obj in terminalreporter.stats.get(''):
            if test_obj.when == 'teardown' and ('skip' not in test_obj.keywords):
                user_influxdb_creds_dict = test_obj.user_properties[0].get('influxdb')
                influxdb_creds_dict.update(dict(influxdb_test_host=user_influxdb_creds_dict['influxdb_host'],
                                                influxdb_test_port=user_influxdb_creds_dict['influxdb_port'],
                                                influxdb_test_user=user_influxdb_creds_dict['influxdb_user'],
                                                influxdb_test_password=user_influxdb_creds_dict['influxdb_password'],
                                                influxdb_test_name=user_influxdb_creds_dict['influxdb_name'],
                                                influxdb_test_values=user_influxdb_creds_dict['influxdb_values'],
                                                influxdb_run_value=user_influxdb_creds_dict['run'],
                                                influxdb_project_value=user_influxdb_creds_dict['project'],
                                                influxdb_version_value=user_influxdb_creds_dict['version']
                                                ))
                break
        return influxdb_creds_dict

    def get_valid_influxdb_values(self, terminalreporter):
        influxdb_creds_dict = self.get_test_influxdb_credentials(terminalreporter)

        db_host = influxdb_creds_dict.get("influxdb_test_host")
        db_port = influxdb_creds_dict.get("influxdb_test_port")
        db_user = influxdb_creds_dict.get("influxdb_test_user")
        db_password = influxdb_creds_dict.get("influxdb_test_password")
        db_name = influxdb_creds_dict.get("influxdb_test_name")
        influxdb_values = influxdb_creds_dict.get("influxdb_test_values")
        run = influxdb_creds_dict.get("influxdb_run_value")
        project = influxdb_creds_dict.get("influxdb_project_value")
        version = influxdb_creds_dict.get("influxdb_version_value")

        influxdb_creds_dict.clear()
        influxdb_creds_dict.update(
            dict(db_host=db_host, db_port=db_port, db_user=db_user, db_password=db_password, db_name=db_name,
                 influxdb_values=influxdb_values, run=run, project=project, version=version))

        return influxdb_creds_dict

    def get_results_dict(self, collected_test_results):
        total_passed = len(collected_test_results.get('passed'))
        total_skipped = len(collected_test_results.get('skipped'))
        total_failed = len(collected_test_results.get('failed'))
        total_disabled = len(collected_test_results.get('xfailed')) + total_skipped

        return dict(passed=total_passed, skipped=total_skipped, failed=total_failed,
                    disabled=total_disabled)



import ast


class TestResultDTO:
    __run = 'UNDEFINED'
    __test = None
    __status = 'disabled'
    __project = 'UNDEFINED'
    __version = 'UNDEFINED'
    __screenshot = None
    __duration_sec = None
    __exception = None
    __merged = None
    __test_result_tag_dict = {}
    __test_result_field_dict = {}

    def set_run(self, run):
        if run != '':
            self.__run = str(run)

    def get_test(self):
        return self.__test

    def set_test(self, test):
        self.__test = str(test)

    def set_status(self, status):
        self.__status = str(status)

    def set_project(self, project):
        if project != '':
            self.__project = str(project)

    def set_version(self, version):
        if version != '':
            self.__version = str(version)

    def set_screenshot(self, screenshot):
        self.__screenshot = str(screenshot)

    def set_duration_sec(self, duration_sec):
        self.__duration_sec = int(duration_sec)

    def set_exception(self, exception):
        self.__exception = str(exception)

    def set_merged(self, is_merged):
        self.__merged = str(is_merged)

    def set_test_result_tag_dict(self, test_result_tag_dict):
        TestResultDTO.__test_result_tag_dict = test_result_tag_dict

    def set_test_result_field_dict(self, test_result_field_dict):
        TestResultDTO.__test_result_field_dict = test_result_field_dict

    def get_test_json(self, measurement_name):
        json_body = [
            {
                "measurement": measurement_name,
                "tags": {
                    "test": self.__test,
                    "run": self.__run,
                    "project": self.__project,
                    "version": self.__version
                },
                "fields": {
                    "status": self.__status,
                    "screenshot": self.__screenshot,
                    "duration_sec": self.__duration_sec,
                    "exception": self.__exception,
                    "merged": self.__merged
                }
            }
        ]
        tags_dict_for_all_tests = TestResultDTO.__test_result_tag_dict
        tags_dict_for_current_test = tags_dict_for_all_tests.get(str(self.get_test()))
        if tags_dict_for_current_test:
            for key in tags_dict_for_current_test:
                json_body[0]['tags'].update({key: tags_dict_for_current_test[key]})
        fields_dict_for_all_tests = TestResultDTO.__test_result_field_dict
        fields_dict_for_current_test = fields_dict_for_all_tests.get(str(self.get_test()))
        if fields_dict_for_current_test:
            for key in fields_dict_for_current_test:
                json_body[0]['fields'].update({key: fields_dict_for_current_test[key]})
        return json_body

    def get_test_json_for_suite_level(self):
        json_body = {
            "test": self.__test,
            "status": self.__status,
            "duration_sec": self.__duration_sec,
            "run": self.__run,
            "project": self.__project,
            "version": self.__version
        }

        return json_body

    def parse_json_to_test_result_dto(self, json_test_result_dto):
        if json_test_result_dto.endswith('\n'):
            json_test_result_dto = json_test_result_dto[:-1]
        json_test_result_dto = ast.literal_eval(json_test_result_dto)
        test_result_dto = TestResultDTO()
        test_result_dto.set_test(json_test_result_dto['test'])
        test_result_dto.set_status(json_test_result_dto['status'])
        test_result_dto.set_duration_sec(json_test_result_dto['duration_sec'])
        test_result_dto.set_run(json_test_result_dto['run'])
        test_result_dto.set_project(json_test_result_dto['project'])
        test_result_dto.set_version(json_test_result_dto['version'])

        return test_result_dto

    def set_tag_values(self, item_nodeid, tags_dict):
        main_dict = TestResultDTO.__test_result_tag_dict
        if item_nodeid in main_dict:
            main_dict[item_nodeid].update(tags_dict)
        else:
            main_dict.update({item_nodeid: tags_dict})

    def set_field_values(self, item_nodeid, fields_dict):
        main_dict = TestResultDTO.__test_result_field_dict
        if item_nodeid in main_dict:
            main_dict[item_nodeid].update(fields_dict)
        else:
            main_dict.update({item_nodeid: fields_dict})

    def set_test_result_data(self, project_name, release_version, run_id_value, report_outcome,
                             merged):
        self.set_project(project_name)
        self.set_version(release_version)
        self.set_run(run_id_value)
        self.set_duration_sec(report_outcome.duration)
        self.set_status(report_outcome.outcome)
        self.set_merged(merged)

    def collect_custom_values(self, influxdb_values):
        custom_values = influxdb_values
        if custom_values and custom_values is not '':
            influxdb_values_dict: dict = ast.literal_eval(str(custom_values))
            self.set_field_values(self.get_test(), influxdb_values_dict['fields']['test_result'])
            self.set_tag_values(self.get_test(), influxdb_values_dict['tags']['test_result'])

    def collect_user_properties(self, request):
        user_properties_tuple = request.node.user_properties
        for var in user_properties_tuple:
            if isinstance(var, dict):
                self.set_field_values(self.get_test(), var)



import os

import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


setuptools.setup(
    name="pytest_influxdb",
    version='0.0.9',
    author="Strike Team",
    author_email="elenaramyan@workfront.com",
    description="Plugin for influxdb and pytest integration.",
    long_description=read('README.rst'),
    url="",
    packages=['pytest_influxdb'],
    classifiers=[
        "Programming Language :: Python :: 3"
    ],
    install_requires=['pytest', 'pytest-xdist', 'influxdb', 'pytest-rerunfailures'],
    entry_points={'pytest11': ['pytest-influxdb-plugin = pytest_influxdb.plugin', ], },
)



from influxdb import InfluxDBClient


class Influxdb_Components:
    __client = None

    def __init__(self, host, port, db_user, db_password, db_name):
        self.__client = InfluxDBClient(host, port, db_user, db_password, db_name)

    def get_client(self):
        return self.__client

    def get_results_by_run(self, measurement_name, run_id):
        return self.__client.query(
            f"SELECT * FROM {measurement_name} WHERE run='{run_id}'")

    def delete_results_by_run(self, measurement_name, run_id):
        return self.__client.query(
            f"DELETE FROM {measurement_name} WHERE run='{run_id}'")

    def write_points(self, json):
        self.__client.write_points(json)


import pytest

from pytest_influxdb.data_manager import DataManager
from pytest_influxdb.influxdb_components import Influxdb_Components
from pytest_influxdb.suite_result_dto import SuiteResultDTO

data_manager = DataManager()
test_result_dto_session_dict = dict()
session_dict = {'test_result_dto_session_dict': test_result_dto_session_dict}
db_measurement_name_for_test = 'test_result'
db_measurement_name_for_suite = 'suite_result'


@pytest.mark.tryfirst
def pytest_runtest_makereport(item, call, __multicall__):
    if item.config.option.pytest_influxdb:
        report = __multicall__.execute()
        setattr(item, "rep_" + report.when, report)
        return report


@pytest.fixture(scope='function', autouse=True)
def test_result(request, pytestconfig, get_influxdb, run, project, version, influxdb_values, db_host, db_port, db_name,
                db_user, db_password, merged):
    test_name = request.node.nodeid
    test_result_dto = data_manager.get_test_result_dto(test_name, session_dict)

    if pytestconfig.getoption('--pytest-influxdb'):
        def send_test_data_to_db():
            # Generating test data
            test_result_dto.set_test_result_data(project, version, run, data_manager.get_report(request), merged)
            test_result_dto.collect_user_properties(test_result_dto, request)
            test_result_dto.collect_custom_values(test_result_dto, influxdb_values)

            send_data_to_db(test_result_dto, get_influxdb)

            data_manager.save_db_data_in_properties(request=request, db_host=db_host, db_port=db_port, db_user=db_user,
                                                    db_password=db_password, db_name=db_name,
                                                    influxdb_values=influxdb_values, run=run, project=project,
                                                    version=version)

        request.addfinalizer(send_test_data_to_db)


def pytest_exception_interact(node, call, report):
    if report.failed:
        test_name = node.nodeid
        test_result_dto = session_dict.get('test_result_dto_session_dict').get(test_name)
        if test_result_dto:
            stack_trace = node.repr_failure(call.excinfo)
            stack_trace = str(stack_trace).replace('"', "'")
            test_result_dto.set_exception(stack_trace)


def pytest_configure(config):
    config.getini("influxdb_run_id")
    config.getini("influxdb_project")
    config.getini("influxdb_version")
    config.getini("influxdb_host")
    config.getini("influxdb_port")
    config.getini("influxdb_user")
    config.getini("influxdb_password")
    config.getini("influxdb_name")
    config.getini("influxdb_project")
    config.getini("influxdb_values")


def pytest_addoption(parser):
    group = parser.getgroup('pytest-influxdb')
    group.addoption("--pytest-influxdb", action="store_true",
                    help="pytest-influxdb: enable influxdb data collection")
    parser.addoption(
        "--influxdb_run_id", action="store", default=None, help="my option: run_id"
    )
    parser.addoption(
        "--influxdb_host", action="store", default=None, help="my option: db_host"
    )
    parser.addoption(
        "--influxdb_port", action="store", default=None, help="my option: db_port"
    )
    parser.addoption(
        "--influxdb_user", action="store", default=None, help="my option: db_username"
    )
    parser.addoption(
        "--influxdb_password", action="store", default=None, help="my option: db_password"
    )
    parser.addoption(
        "--influxdb_name", action="store", default=None, help="my option: db_name"
    )
    parser.addoption(
        "--influxdb_project", action="store", default=None, help="my option: project name"
    )
    parser.addoption(
        "--influxdb_version", action="store", default=None, help="my option: version"
    )
    parser.addoption(
        "--influxdb_values", action="store", default=None, help="my option: values"
    )
    parser.addoption(
        "--influxdb_merged", action="store_true", default=None, help="my option: merged"
    )
    parser.addini(
        'influxdb_run_id',
        help='my option: run_id')
    parser.addini(
        'influxdb_project',
        help='my option: project')
    parser.addini(
        'influxdb_version',
        help='my option: version')
    parser.addini(
        'influxdb_host',
        help='my option: db_host')
    parser.addini(
        'influxdb_port',
        help='my option: db_port')
    parser.addini(
        'influxdb_name',
        help='my option: db_name')
    parser.addini(
        'influxdb_user',
        help='my option: db_user')
    parser.addini(
        'influxdb_password',
        help='my option: db_password')
    parser.addini(
        'influxdb_values',
        help='my option: db_values')


@pytest.fixture
def run(request, pytestconfig):
    run_id = request.config.getoption("--influxdb_run_id")
    if not run_id:
        run_id = pytestconfig.getini('influxdb_run_id')
    return run_id


@pytest.fixture
def db_host(request, pytestconfig):
    db_host = request.config.getoption("--influxdb_host")
    if not db_host:
        db_host = pytestconfig.getini("influxdb_host")
    return db_host


@pytest.fixture
def db_port(request, pytestconfig):
    db_port = request.config.getoption("--influxdb_port")
    if not db_port:
        db_port = pytestconfig.getini("influxdb_port")
    if not db_port:
        db_port = 8086
    return db_port


@pytest.fixture
def db_user(request, pytestconfig):
    db_user = request.config.getoption("--influxdb_user")
    if not db_user:
        db_user = pytestconfig.getini("influxdb_user")
    return db_user


@pytest.fixture
def db_password(request, pytestconfig):
    db_password = request.config.getoption("--influxdb_password")
    if not db_password:
        db_password = pytestconfig.getini("influxdb_password")
    return db_password


@pytest.fixture
def db_name(request, pytestconfig):
    db_name = request.config.getoption("--influxdb_name")
    if not db_name:
        db_name = pytestconfig.getini("influxdb_name")
    return db_name


@pytest.fixture
def project(request, pytestconfig):
    project_name = request.config.getoption("--influxdb_project")
    if not project_name:
        project_name = pytestconfig.getini('influxdb_project')
    return project_name


@pytest.fixture
def version(request, pytestconfig):
    version = request.config.getoption("--influxdb_version")
    if not version:
        version = pytestconfig.getini('influxdb_version')
    return version


@pytest.fixture
def influxdb_values(request, pytestconfig):
    values = request.config.getoption("--influxdb_values")
    if not values:
        values = pytestconfig.getini("influxdb_values")
    return values


@pytest.fixture()
def merged(request):
    return request.config.getoption("--influxdb_merged")


@pytest.fixture()
def get_influxdb(pytestconfig, db_host, db_port, db_name, db_user, db_password):
    if pytestconfig.getoption('--pytest-influxdb'):
        influxdb_components = Influxdb_Components(db_host, db_port, db_user, db_password, db_name)
        return influxdb_components


# Fixture for generating a screenshot from conftest
@pytest.fixture()
def screenshot_url(request, pytestconfig):
    if pytestconfig.getoption('--pytest-influxdb'):
        current_test = request.node.nodeid

        def _foo(*args, **kwargs):
            test_result_dto = session_dict.get('test_result_dto_session_dict').get(current_test)
            if pytestconfig.getoption('--pytest-influxdb'):
                test_result_dto.set_screenshot(args[0])
            return (args, kwargs)

        return _foo


@pytest.hookimpl(hookwrapper=True)
def pytest_terminal_summary(config, terminalreporter):
    yield
    if config.getoption('--influxdb-pytest') and len(terminalreporter.stats) > 0:
        global_values = data_manager.get_valid_influxdb_values(terminalreporter)
        influxdb_components = Influxdb_Components(global_values.get("db_host"), global_values.get("db_port"),
                                                  global_values.get("db_user"), global_values.get("db_password"),
                                                  global_values.get("db_name"))

        suite_result_dto = SuiteResultDTO().get_suite_result_dto(terminalreporter, global_values)
        suite_result_dto.set_suite_custom_values(global_values.get("influxdb_values"))


def send_data_to_db(test_result_dto, influxdb):
    ##Sending the test results to influxdb
    test_json = test_result_dto.get_test_json(db_measurement_name_for_test)
    data_manager.get_test_influxdb_components(influxdb, session_dict).write_points(test_json)
